//---------------------------------------------------------------------------
// Written by Chris Maunder (chrismaunder@codeguru.com)
// Extended by Alexander Bischofberger (bischofb@informatik.tu-muenchen.de)
// Copyright (c) 1998.
//
// This code may be used in compiled form in any way you desire. This
// file may be redistributed unmodified by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. If 
// the source code in  this file is used in any commercial application 
// then a simple email would be nice.
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability if it causes any damage whatsoever.
// It's free - so you get what you pay for.
//---------------------------------------------------------------------------

#if !defined(AFX_TColorPopup_H__D0B75902_9830_11D1_9C0F_00A0243D1382__INCLUDED_)
#define AFX_TColorPopup_H__D0B75902_9830_11D1_9C0F_00A0243D1382__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000


// CColorPopup messages
#define CPN_SELCHANGE        WM_USER + 1001        // Color Picker Selection change
#define CPN_DROPDOWN         WM_USER + 1002        // Color Picker drop down
#define CPN_CLOSEUP          WM_USER + 1003        // Color Picker close up
#define CPN_SELENDOK         WM_USER + 1004        // Color Picker end OK
#define CPN_SELENDCANCEL     WM_USER + 1005        // Color Picker end (cancelled)

class CColorPicker;

typedef struct
{
    COLORREF crColor;
    TCHAR    *szName;
} ColorTableEntry;

class CColorPopup : public CWnd
{
public:
    CColorPopup();
    CColorPopup(CPoint		point, 
				 COLORREF	crColor, 
				 CWnd*		pParentWnd,
                 LPCTSTR	szDefaultText = NULL, 
				 LPCTSTR	szCustomText = NULL);

    virtual ~CColorPopup();

    void Initialize();

    BOOL			Create(CPoint		point, 
						   COLORREF		crColor, 
						   CWnd*		pParentWnd, 
						   LPCTSTR		szDefaultText = NULL, 
						   LPCTSTR		szCustomText = NULL);

    virtual BOOL	PreTranslateMessage(MSG* pMsg);

protected:
    BOOL			GetCellRect(int nIndex, const LPRECT& rect);
    void			FindCellFromColor(COLORREF crColor);
    void			SetWindowSize();
    void			CreateToolTips();
    void			ChangeSelection(int nIndex);
    void			EndSelection(int nMessage);
    void			DrawCell(CDC* pDC, int nIndex);

    COLORREF		GetColor(int nIndex);
    LPCTSTR			GetColorName(int nIndex);
    int				GetIndex(int row, int col) const;
    int				GetRow(int nIndex) const;
    int				GetColumn(int nIndex) const;

    afx_msg void	OnNcDestroy();
    afx_msg void	OnLButtonUp(UINT nFlags, CPoint point);
    afx_msg void	OnPaint();
    afx_msg void	OnMouseMove(UINT nFlags, CPoint point);
    afx_msg void	OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
    afx_msg BOOL	OnQueryNewPalette();
    afx_msg void	OnPaletteChanged(CWnd* pFocusWnd);
	afx_msg void	OnKillFocus(CWnd* pNewWnd);
	afx_msg void	OnActivateApp(BOOL bActive, HTASK hTask);

    DECLARE_MESSAGE_MAP()

private:
	static ColorTableEntry m_crColors[];

    int            m_nNumColors;
    int            m_nNumColumns;
    int            m_nNumRows;
    int            m_nBoxSize;
    int            m_nMargin;
    int            m_nCurrentSel;
    int            m_nChosenColorSel;
    CString        m_strDefaultText;
    CString        m_strCustomText;
    CRect          m_CustomTextRect;
    CRect          m_DefaultTextRect;
    CRect          m_WindowRect;
    CFont          m_Font;
    CPalette       m_Palette;
    COLORREF       m_crInitialColor;
    COLORREF       m_crColor;
    CToolTipCtrl   m_oToolTip;
    CWnd*          m_pParent;

    BOOL           m_bChildWindowVisible;

};

//---------------------------------------------------------------------------
// GetColor
//---------------------------------------------------------------------------
inline COLORREF		
CColorPopup::GetColor(int nIndex)
{ 
	return m_crColors[nIndex].crColor;
}

    
//---------------------------------------------------------------------------
// GetColorName
//---------------------------------------------------------------------------
inline LPCTSTR
CColorPopup::GetColorName(int nIndex)
{
	return m_crColors[nIndex].szName;
}


#endif // !defined(AFX_TCOLORPOPUP_H__D0B75902_9830_11D1_9C0F_00A0243D1382__INCLUDED_)
